uses dos,crt;
{$I serial.pas}
{$I 8259.pas}
{$I hprint}
{$I beepy}
{$I 4I23Low}

const
FirstPort = 0;
LastPort = 3;
SerPorts : array[FirstPort..LastPort] of word =(com1,com2,com3,com4);
FIFOSize = 20000;
ControlCInt = $1B;
type

RecCharFIFO = record
  PutPointer : word;
  GetPointer : word;
  DataBuffer : array [0..FIFOSize] of byte;
  TagBuffer  : array [0..FIFOSize] of byte;
end;

var

SharedIntStatusReg : word;
SharedIntMaskReg : word;
RecFIFO : RecCharFIFO;
ComInterrupt : byte;
ComVectorSave : pointer;
ControlCVectorSave : pointer;
TheBaudRate : longint;

function RecFIFOInc(pt : word) : word;
begin
  pt := pt + 1;                { watch out for fifo size > 64k ! }
  if (pt > FIFOSize) then pt := 0;
  RecFifoInc := pt;
end;

procedure InitRecFIFO;
begin
  with RecFIFO do
  begin
    PutPointer := 0;
    GetPointer := 0;
  end;
end;

function RecFIFOHasData : boolean;
begin
  RecFIFOHasData := false;
  with RecFIFO do
  begin
    if GetPointer <> PutPointer then RecFIFOHasData := true;
  end;
end;

function RecFIFOIsFull : boolean;
begin
  RecFIFOIsFull := false;
  with RecFIFO do
  begin
    { if hypothetical -put- would smash -get- data then fIFO is full! }
    if RecFIFOInc(PutPointer) = GetPointer then RecFIFOIsFull := true;
  end;
end;

procedure RecFIFOGet(var data,tag : byte);
begin
  if RecFIFOHasData then
  begin
    with RecFIFO do
    begin
      data := DataBuffer[GetPointer];
      tag := TagBuffer[GetPointer];
      GetPointer := RecFIFOInc(GetPointer);
    end;
  end;
end;

procedure RecFIFOPeek(var data : byte);
begin
  with RecFIFO do data := DataBuffer[GetPointer];
end;

procedure RecFIFOPut(data,tag :byte);
begin
  if (not RecFIFOIsFull) then
  begin
    with RecFIFO do
    begin
      DataBuffer[PutPointer]  := data;
      TagBuffer[PutPointer]  := tag;
      PutPointer := RecFIFOInc(PutPointer);
    end;
  end;
end;

procedure InitVars;
begin
  InitRecFIFO;
  TheBaudRate := 9600;
  ComInterrupt := 4;
  SharedIntMaskReg := SerPorts[FirstPort] or $8000;
  SharedIntStatusReg := SerPorts[FirstPort] or $8000;
end;

procedure SharedISR;
interrupt;
var
ch,chmask,dummy : byte;
baseport : word;
begin
                             { First disable all shared interrupt sources to avoid new edges }
                             { the shared interupt tri-state drive enabled }
  { As long as any interrupts are pending, loop to extract them from the UARTS }
  SharedIntMaskReg := $10; {Disable shared interrupt sources}

  while (port[SharedIntStatusReg] and $1F) <> 0 do
  begin
    { This of course assumes that all 4 channels ints are or'ed }

    chmask := $01;
    for ch := FirstPort to LastPort do
    begin
      if (port[SharedIntStatusReg] and chmask) <> 0 then { if this channels IRQ is asserted ...}
      begin
        baseport := SerPorts[ch];
        if (port[baseport+IntId] and $7) = 4 then { valid receive char or FIFO timeout on '550 }
        begin
          RecFIFOPut(port[baseport + RecReg],ch);
        end
        else  { clear error status }
        begin
          dummy := port[baseport +LineStat];
        end;
      end;
      chmask := chmask shl 1;
    end;
  end;
  SharedIntMaskReg := $1F; {Re-enable shared interrupt sources}
  IssueEOI;
end;

procedure ControlCISR;
interrupt;
begin
  DisableInterrupts;
  DoBeep(500,3);
  DoBeep(700,6);
  EnableInterrupts;
end;

procedure DisplayLoop;
var
data,ch: byte;
count : array [FirstPort..LastPort] of longint;
begin
  for ch := FirstPort to LastPort do count[ch] := 0;
  ClrScr;
  while not keypressed do
  begin
    if RecFIFOHasData then
    begin
      RecFIFOGet(data,ch);
      GotoXY(1,(ch+1)*2);
      count[ch] := count[ch] +1;
      write('ch ',ch:0,' ');
      write(count[ch]:8,' ');  { dont need to pad with spaces since the numbers}
      hexprint(data,2);        { should just increment... }
    end
  end;
end;

procedure InitPorts;
var
portindex : byte;
comport :word;
begin
  for portindex := FirstPort to LastPort do
  begin
    comport := SerPorts[portindex];
    DefaultComInit(comport);
    SetBaudRate(comport,TheBaudRate);
    EnableIntDriver(comport);
    SetUpForRecInt(comport);
  end;
end;

procedure InstallIntStuff;
var
vectorsave : pointer;

begin
  DisableInterrupts;
  MaskInterrupt(ComInterrupt);                 { make sure interrupts are disabled }
  GetHardIntVec(ComInterrupt,ComVectorSave);   { save old vector  }
  SetHardIntVec(ComInterrupt,@SharedISR);      { install ISR vector }
  GetIntVec(ControlCInt,ControlCVectorSave);
  SetIntVec(ControlCInt,@ControlCISR);         { install control C handler }
  UnMaskInterrupt(ComInterrupt);               { and away we go... }
  EnableInterrupts;
end;

procedure ShutDown;
begin
  DisableInterrupts;
  MaskInterrupt(ComInterrupt);                 { Shut the door before we leave... }
  SetHardIntVec(ComInterrupt,ComVectorSave);   { restore old vector }
  SetIntVec(ControlCInt,ControlCVectorSave);   { restore old control C handler }
  EnableInterrupts;
  port[SharedIntMaskReg] := $0;                { Disable shared int ts driver }

end;


procedure Install;
begin
  InitVars;
  InitPorts;
  port[SharedIntMaskReg] := $1F;               { enable shared ints on all channels}
  InstallIntStuff;
end;

begin
  If Is4I23(SerPorts[FirstPort]) then
  begin
    Install;
    DisplayLoop;
    Shutdown;
  end
  else
  writeln('This program only runs on Mesa 4I23 cards');
end.
