{$R-}    {make sure were home on the range}
{$I-}    {I/O checking on }
{$S-}    {make sure the stack doesn't deck us}


uses DOS;
{$I 8259.pas}                         { include low level PC interrupt stuff }
{$I Serial.pas}                       { include low level serial routines }
{$I Beepy.pas}                        { include speaker beep stuff }

const
{ the hardware interrupt we are using for com1 or com2 }


DivisorFor9600 = 12;
DivisorFor112K = 1;
DivisorFor150  = 768;
DivisorFor75   = 1536;
DivisorFor15   = 7680;

var
ComVectorSave : pointer;
ComInterrupt : byte;
SSSave : word;
SPSave : word;
NewSS  : word;
NewSP  : word;
IRQNum : byte;
DoIRQTest : boolean;
DoHandShakeTest : boolean;
IRQMask : byte;
TheComPort : word;
SendChar : byte;
RecvChar : byte;
AChar : char;

startwink,startsecond,startminute,starthour : word;
endwink,endsecond,endminute,endhour : word;
delwink,delsecond,delminute,delhour : word;


procedure HexWordPrint(theWord : word);
var
shiftCount : integer;
const
hexChars : array[0..15] of char = ('0','1','2','3','4','5','6','7',
                                   '8','9','A','B','C','D','E','F');
begin
shiftCount := 12;
  while shiftCount >= 0 do
  begin
    write(hexChars[((theWord shr shiftCount) and $000F)]);
    shiftCount := shiftCount - 4;
  end;
end;

function HexWordRead(hexstring : string; var hexnumber : word): boolean;
var
hexread : boolean;
hindex : byte;
charVal : integer;
placeVal : integer;
const
hexChars : string = '0123456789ABCDEF';
begin
  hexNumber :=0;
  HexRead := true;
  for hindex := 1 to length(hexstring) do
  begin
    hexNumber := HexNumber * $10;
    charval := pos(upcase(hexstring[hindex]),hexChars) -1;
    if charval < 0 then
    begin
      HexRead := false;
    end
    else
    begin
      hexnumber := hexnumber + charVal;
    end;
  end;
end;

procedure Delay(del : word);
var
iloop,oloop : word;

begin
  for oloop := 0 to del do
  begin
    for iloop := 1 to 300 do
    begin
    end;
  end;
end;

procedure StartTimer;

begin
  GetTime(starthour,startminute,startsecond,startwink);
end;

procedure StopTimer;
begin
  GetTime(endhour,endminute,endsecond,endwink);
end;

function ReadTimer : real;
begin
  delwink   := (endwink-startwink);
  delsecond := (endsecond-startsecond);
  delminute := (endminute-startminute);
  delhour   := (endhour-starthour);
  if delwink   > 32767 then
    begin
      delwink   := 100 + delwink;
      delsecond := delsecond - 1;
    end;
  if delsecond > 32767 then
   begin
     delsecond := 60 + delsecond;
     delminute := delminute - 1;
   end;
  if delminute > 32767 then
    begin
      delminute := 60 + delminute;
      delhour := delhour - 1;
    end;
  if delhour > 32767 then delhour  := 24 + delhour;
  ReadTimer := (delhour*360000+delminute*6000+delsecond*100+delwink);
end; { ReadTimer }


function GetParm : boolean;
var retcode : integer;
begin
  GetParm := true;
  DoIRQTest := false;
  DoHandShakeTest := true;
  IRQNum := 0;
  if (not HexWordRead(ParamStr(1),TheComPort)) or (TheComPort > $3FF) then
  begin
    write('Could not extract valid port address from ');
    writeln(ParamStr(1));
    GetParm := false;
  end;
  if length(ParamStr(2)) <> 0 then
  begin
    val(ParamStr(2),IRQNum,retcode);
    if (IRQNum > 1) and (IRQNum < 8) then
    begin
      DoIRQTest := true;
      IRQMask := $01 shl IRQNum;
    end
    else
    begin
      writeln('Invalid IRQ number -- must be between 2 and 7' );
      GetParm := false;
    end;
  end;
  if length(ParamStr(3)) <> 0 then
  begin
    DoHandShakeTest := false;
  end;
end;

procedure SerialISR;
var
status : boolean;
begin
  status := GetRecIntChar(TheComPort,AChar);
  IssueEOI;
end;

procedure COMInterruptServiceRoutine;
interrupt;
begin
  asm
    MOV AX,SS          { save old SS and SP }
    MOV SSSave,AX
    MOV AX,SP
    MOV SPSave,AX
    MOV AX,NewSS       { install new SS and SP }
    MOV SS,AX
    MOV AX,NewSP
    MOV SP,AX
  end;
  SerialISR;
  asm
    MOV AX,SSSave      { restore old SS and SP (we hope) }
    MOV SS,AX
    MOV AX,SPSave
    MOV SP,AX
  end;
end; { InterruptServiceRoutine }

procedure InitComPortForInt(comport: word);
begin
  DefaultComInit(comport);
  SetBRDivisor(comport,DivisorFor9600);
  EnableIntDriver(comport);
  SetUpForRecInt(comport);
  UnMaskInterrupt(ComInterrupt);               { and away we go... }
  EnableInterrupts;
end;

procedure InitComPortForPoll(comport: word; divisor: word);
begin
  DefaultComInit(comport);
  SetBRDivisor(comport,divisor);
  Delay(50);
  DefaultComInit(comport);                     { just to make sure!}
  Delay(50);
  DisableIntDriver(comport);
end;

procedure SetRTS(comport : word);
var
tdata : byte;

begin
  tdata := port[comport+ModemCont];
  tdata := (tdata or RTSBit);
  port[comport+ModemCont] := tdata;
end;

procedure SetDTR(comport : word);
var
tdata : byte;

begin
  tdata := port[comport+ModemCont];
  tdata := (tdata or DTRBit);
  port[comport+ModemCont] := tdata;
end;

procedure ClearDTR(comport : word);
var
tdata : byte;

begin
  tdata := port[comport+ModemCont];
  tdata := (tdata and ($FF xor DTRBit));
  port[comport+ModemCont] := tdata;
end;


procedure ClearRTS(comport : word);
var
tdata : byte;

begin
  tdata := port[comport+ModemCont];
  tdata := (tdata and ($FF xor RTSBit));
  port[comport+ModemCont] := tdata;
end;

function CDandDSRSet(comport : word) : boolean;
var
tdata : byte;

begin
  CDandDSRSet := true;
  tdata := port[comport+ModemStat];
  tdata := tdata and (CDBit or DSRBit);
  if tdata <> (CDBit or DSRBit) then CDandDSRSet := false;
end;

function RIandCTSSet(comport : word) : boolean;
var
tdata : byte;

begin
  RIandCTSSet := true;
  tdata := port[comport+ModemStat];
  tdata := tdata and (RIBit or CTSBit);
  if tdata <> (RIBit or CTSBit) then RIandCTSSet := false;
end;


function RIandCTSClear(comport : word) : boolean;
var
tdata : byte;

begin
  RIandCTSClear := true;
  tdata := port[comport+ModemStat];
  tdata := tdata and (RIBit or CTSBit);
  if tdata <> 0 then RIandCTSClear := false;
end;

function CDandDSRClear(comport : word) : boolean;
var
tdata : byte;

begin
  CDandDSRClear := true;
  tdata := port[comport+ModemStat];
  tdata := tdata and (CDBit or DSRBit);
  if tdata <> 0 then CDandDSRClear := false;
end;

function BitSet(comreg : word; bitmask: byte) : boolean;
var
tdata : byte;

begin
  BitSet := true;
  tdata := port[comreg];
  tdata := tdata and (bitmask);
  if tdata <> (bitmask) then BitSet := false;
end;

function BitClr(comreg : word; bitmask: byte) : boolean;
var
tdata : byte;

begin
  BitClr := true;
  tdata := port[comreg];
  tdata := tdata and (bitmask);
  if tdata <> 0 then BitClr := false;
end;


function GetComChar(comPort : word; var cchar : byte) : boolean;
var
loops : longint;
tdata : byte;

begin
  tdata := 0;
  loops := 0;
  GetComChar := true;
  while (tdata = 0) and (loops < 100000) do
  begin
    tdata := port[comPort + LineStat] and DataReady;
    loops := loops +1;
  end;
  if loops >= 90000 then GetComChar := false;
  cchar := (port[comPort + RecReg]);
end;

function PutComChar(comPort : word; cchar : byte) : boolean;
var
loops : longint;
tdata : byte;

begin
  tdata := 0;
  loops := 0;
  PutComChar := true;
  while (tdata = 0) and (loops < 100000) do
  begin
    tdata :=port[comPort + LineStat] and THRE;
    loops := loops +1;
  end;
  if loops >= 90000 then PutComChar := false;
  port[comPort + TrHReg] := (cchar);
end;


procedure BadBeeps;
begin
  BeepOn(800);
  Delay(200);
  BeepOn(1500);
  Delay(100);
  BeepOff;
end;

procedure HappyBeeps;
begin
  BeepOn(1000);
  Delay(50);
  BeepOn(1200);
  Delay(25);
  BeepOn(1000);
  Delay(50);
  BeepOn(1200);
  Delay(25);
  BeepOn(1000);
  Delay(50);
  BeepOn(1200);
  Delay(25);
  BeepOff;
end;

procedure Bumout;
begin
  BadBeeps;
  Halt(2);
end;

procedure RemSLooptest(comport : word);
var
sdata, rdata : byte;
loopctr : word;
begin
  writeln('Doing serial data loopback test @ 112K baud');
  for loopctr := 0 to 5000 do                 { about .5 second at 112K }
  begin
    sdata := lo(loopctr);
    if not PutComChar(comport,sdata) then
    begin
      writeln('Serial data loopback failure >> TRANSMIT TIMEOUT');
      Bumout;
    end;
    if not GetComChar(comport,rdata) then
    begin
      writeln('Serial data loopback failure >> RECEIVE TIMEOUT');
      Bumout;
    end;
    if sdata <> rdata then
    begin
      writeln('Serial data loopback failure >> DATA MISMATCH');
      Bumout;
    end;
  end;
  writeln('Loopback test ok!');
end;

procedure RemBaudRateCheck(comport : word);
var
sdata, rdata : byte;
loopctr : word;
terror : integer;
begin
  writeln('Doing baud rate check @ 150 baud');
  {Should do local explicit port set-up because of sensitivity to any timing changes! }
  {mebe layder...}
  InitComPortForPoll(comport,DivisorFor150);
  StartTimer;
  for loopctr := 1 to 50 do                { about 3 second at 150 baud }
  begin
    sdata := $AA;
    if not PutComChar(comport,sdata) then
    begin
      writeln('Baud Check data loopback failure >> TRANSMIT TIMEOUT');
      Bumout;
    end;
    if not GetComChar(comport,rdata) then
    begin
      writeln('Baud check data loopback failure >> RECEIVE TIMEOUT');
      Bumout;
    end;
    if sdata <> rdata then
    begin
      writeln('Baud check data loopback failure >> DATA MISMATCH');
      Bumout;
    end;
  end;
  StopTimer;
  {What a HORRRRRIBLE HACK! }
  terror := trunc(350 - (ReadTimer));
  if abs(terror) > 20 then
  begin
    writeln('Minor Baud rate error  >> TIMING WRONG... ',ReadTimer);
    writeln('time error is ',terror,' hundredths of a second out of 3 seconds');
  end;
  if abs(terror) > 56 then
  begin
    writeln('Major Baud rate error  >> TIMING WRONG... ',ReadTimer);
    writeln('time error is ',terror,' hundredths of a second out of 3 seconds');
    Bumout;
  end;
  writeln('baud rate test ok! -- ',trunc(ReadTimer));
end;

procedure GenerateInt(comport: word);
begin
  if not PutComChar(comport,$55) then
  begin
    writeln('Interrupt test failure >> TRANSMIT TIMEOUT ');
    Bumout;
  end;
  WaitForXmitDone(comport);
end;


function WhatInt: byte;
begin
  if IrqNum > 8 then
  begin
    WhatInt := ReadIIR2;
    IssueEOI2
  end
  else
  begin
    WhatInt := ReadIIR;
    IssueEOI;
  end;
end;

procedure CheckInterrupt(comport : word);
var
begstat : byte;
endstat : byte;
response : string;
begin
  writeln('Doing IRQ check');
  begstat := WhatInt;          { clear any existing IRQs }
  InitComPortForPoll(TheComPort,DivisorFor112K);
  ForceIRQLow(TheComPort);     { enable IRQ driver / force IRQ low }
  SetUpForXmitInt(TheComPort); { Enable XMIT IRQ }
  delay(100);                  { wait... for badness to occur }
  begstat := WhatInt;          { check for IRQs }
  if begstat and IRQMask <> 0 then
  begin
    writeln('Interrupt test failure -- IRQ ',IRQNum,' >>> false high');
    InitComPortForPoll(TheComPort,DivisorFor112K);
    Bumout;
  end;
  GenerateInt(TheComPort);
  begstat := WhatInt;          { check for IRQs }
  if begstat and IRQMask = 0 then
  begin
    writeln('Interrupt test failure >>> NO IRQ ',IRQNum);
    InitComPortForPoll(TheComPort,DivisorFor112K);
    Bumout;
  end;
  begin
    writeln('Interrupt generation ok');
  end;
  InitComPortForPoll(TheComPort,DivisorFor112K);
end;

procedure RemHandshakeTest(comport : word);
var
loops : byte;

begin
  writeln('Doing handshake line test');
  for loops := 0 to 24 do
  begin
    SetDTR(comport);
    Delay(2);
    if not CDandDSRSet(comport) then
    begin
      writeln('DTR to CD and DSR handshake failure >> NOT HIGH');
      Bumout;
    end;
    ClearDTR(comport);
    Delay(2);
    if not CDandDSRClear(comport) then
    begin
      writeln('DTR to CD and DSR handshake failure >> NOT LOW');
      Bumout;
    end;
    SetRTS(comport);
    Delay(2);
    if not RIandCTSSet(comport) then
    begin
      writeln('RTS to RI and CTS handshake failure >> NOT HIGH');
      Bumout;
    end;
    ClearRTS(comport);
    Delay(2);
    if not RIandCTSClear(comport) then
    begin
      writeln('RTS to RI and CTS handshake failure >> NOT LOW');
      Bumout;
    end;
  end;
  writeln('Handshake test ok!');
end;


procedure Usage;
begin
  writeln;
  writeln('STEST Serial port test program');
  writeln;
  writeln('USAGE: STEST XXX [I] [H]');
  writeln('Where XXX is the hexadecimal serial port address');
  writeln('I is the optional IRQ number to test');
  writeln('H parameter disables the handshake line test if present');
  halt(2);
end;

procedure Test;
begin
  writeln;
  DisableInterrupts;
  if ComPortThere(TheComPort) then
  begin
    EnableInterrupts;
    write('Testing COM port at ');
    HexWordPrint(TheComPort);
    PrintComPortType(TheComPort);
    writeln;
    InitComPortForPoll(TheComPort,DivisorFor112K);
    RemSloopTest(TheComPort);
    RemBaudRateCheck(TheComPort);
    if DoIRQTest then CheckInterrupt(TheComPort);
    if DoHandShakeTest then  RemHandShakeTest(TheComPort);
    writeln ('Port test ok!!');
    HappyBeeps;
    Halt(0);
  end
  else
  begin
    EnableInterrupts;
    write('No COM port found at address ');
    HexWordPrint(TheComPort);
    writeln;
    Bumout;
  end;
end;


begin { main }
  if length(ParamStr(1)) = 0 then Usage else
  begin
    if not GetParm then Usage else Test;
  end;
end. { main };
